@@ -274,10 +274,29 @@ enum ie_type {
 	IE_TYPE_OWE_DH_PARAM                         = 256 + 32,
 	IE_TYPE_PASSWORD_IDENTIFIER                  = 256 + 33,
 	IE_TYPE_GLK_GCR_PARAMETER_SET                = 256 + 34,
+	IE_TYPE_HE_CAPABILITIES                      = 256 + 35,
+	IE_TYPE_HE_OPERATION                         = 256 + 36,
+	IE_TYPE_UORA_PARAMETER_SET                   = 256 + 37,
+	IE_TYPE_MU_EDCA_PARAMETER_SET                = 256 + 38,
+	IE_TYPE_SPATIAL_REUSE_PARAMETER_SET          = 256 + 39,
+	/* Reserved 40 */
+	IE_TYPE_NDP_FEEDBACK_REPORT_PARAMETER        = 256 + 41,
+	IE_TYPE_BSS_COLOR_CHANGE_ANNOUNCEMENT        = 256 + 42,
+	IE_TYPE_QUIET_TIME_PERIOD                    = 256 + 43,
 	IE_TYPE_VENDOR_SPECIFIC_REQUEST              = 256 + 44,
+	IE_TYPE_ESS_REPORT                           = 256 + 45,
+	IE_TYPE_OPS                                  = 256 + 46,
+	IE_TYPE_HE_BSS_LOAD                          = 256 + 47,
 	IE_TYPE_MAX_CHANNEL_SWITCH_TIME              = 256 + 52,
 	IE_TYPE_ESTIMATED_SERVICE_PARAMETERS_OUT     = 256 + 53,
 	IE_TYPE_OCI                                  = 256 + 54,
+	IE_TYPE_MULTIPLE_BSSID_CONFIGURATION         = 256 + 55,
+	/* Reserved 56 */
+	IE_TYPE_KNOWN_BSSID                          = 256 + 57,
+	IE_TYPE_SHORT_SSID_LIST                      = 256 + 58,
+	IE_TYPE_HE_6GHZ_BAND_CAPABILITIES            = 256 + 59,
+	IE_TYPE_UL_MU_POWER_CAPABILITIES             = 256 + 60,
+	/* Reserved 61 - 87 */
 	IE_TYPE_MSCS_DESCRIPTOR                      = 256 + 88,
 	IE_TYPE_SUPPLEMENTAL_CLASS_2_CAPABILITIES    = 256 + 90,
 	IE_TYPE_REJECTED_GROUPS                      = 256 + 92,
@@ -290,7 +309,9 @@ enum ie_type {
 enum ie_vendor_wfa_oi_type {
 	IE_WFA_OI_HS20_INDICATION = 0x10,
 	IE_WFA_OI_OSEN = 0x12,
+	IE_WFA_OI_OWE_TRANSITION = 0x1c,
 	IE_WFA_OI_ROAMING_SELECTION = 0x1d,
+	IE_WFA_OI_CONFIGURATOR_CONNECTIVITY = 0x1e,
 };
 
 enum ie_advertisement_id {
@@ -308,8 +329,14 @@ enum ie_rsn_cipher_suite {
 	IE_RSN_CIPHER_SUITE_TKIP		= 0x0004,
 	IE_RSN_CIPHER_SUITE_CCMP		= 0x0008,
 	IE_RSN_CIPHER_SUITE_WEP104		= 0x0010,
-	IE_RSN_CIPHER_SUITE_BIP			= 0x0020,
+	IE_RSN_CIPHER_SUITE_BIP_CMAC		= 0x0020,
 	IE_RSN_CIPHER_SUITE_NO_GROUP_TRAFFIC	= 0x0040,
+	IE_RSN_CIPHER_SUITE_GCMP		= 0x0080,
+	IE_RSN_CIPHER_SUITE_GCMP_256		= 0x0100,
+	IE_RSN_CIPHER_SUITE_CCMP_256		= 0x0200,
+	IE_RSN_CIPHER_SUITE_BIP_GMAC		= 0x0400,
+	IE_RSN_CIPHER_SUITE_BIP_GMAC_256	= 0x0800,
+	IE_RSN_CIPHER_SUITE_BIP_CMAC_256	= 0x1000,
 };
 
 enum ie_rsn_akm_suite {
@@ -334,26 +361,67 @@ enum ie_rsn_akm_suite {
 	IE_RSN_AKM_SUITE_OSEN			= 0x40000,
 };
 
-#define IE_AKM_IS_SAE(akm) \
-	((akm == IE_RSN_AKM_SUITE_SAE_SHA256) || \
-	(akm == IE_RSN_AKM_SUITE_FT_OVER_SAE_SHA256))
+static inline bool IE_AKM_IS_SAE(uint32_t akm)
+{
+	return akm & (IE_RSN_AKM_SUITE_SAE_SHA256 |
+			IE_RSN_AKM_SUITE_FT_OVER_SAE_SHA256);
+}
 
-#define IE_AKM_IS_FT(akm) \
-	(akm & (IE_RSN_AKM_SUITE_FT_OVER_8021X | \
-		IE_RSN_AKM_SUITE_FT_USING_PSK | \
-		IE_RSN_AKM_SUITE_FT_OVER_SAE_SHA256 | \
-		IE_RSN_AKM_SUITE_FT_OVER_8021X_SHA384 | \
-		IE_RSN_AKM_SUITE_FT_OVER_FILS_SHA256 | \
-		IE_RSN_AKM_SUITE_FT_OVER_FILS_SHA384))
+static inline bool IE_AKM_IS_FT(uint32_t akm)
+{
+	return akm & (IE_RSN_AKM_SUITE_FT_OVER_8021X |
+			IE_RSN_AKM_SUITE_FT_USING_PSK |
+			IE_RSN_AKM_SUITE_FT_OVER_SAE_SHA256 |
+			IE_RSN_AKM_SUITE_FT_OVER_8021X_SHA384 |
+			IE_RSN_AKM_SUITE_FT_OVER_FILS_SHA256 |
+			IE_RSN_AKM_SUITE_FT_OVER_FILS_SHA384);
+}
 
-#define IE_AKM_IS_FILS(akm) \
-	((akm == IE_RSN_AKM_SUITE_FILS_SHA256) || \
-	 (akm == IE_RSN_AKM_SUITE_FILS_SHA384) || \
-	 (akm == IE_RSN_AKM_SUITE_FT_OVER_FILS_SHA256) || \
-	 (akm == IE_RSN_AKM_SUITE_FT_OVER_FILS_SHA384))
+static inline bool IE_AKM_IS_FILS(uint32_t akm)
+{
+	return (akm) & (IE_RSN_AKM_SUITE_FILS_SHA256 |
+			IE_RSN_AKM_SUITE_FILS_SHA384 |
+			IE_RSN_AKM_SUITE_FT_OVER_FILS_SHA256 |
+			IE_RSN_AKM_SUITE_FT_OVER_FILS_SHA384);
+}
+
+static inline bool IE_AKM_IS_8021X(uint32_t akm)
+{
+	return akm & (IE_RSN_AKM_SUITE_8021X |
+			IE_RSN_AKM_SUITE_FT_OVER_8021X |
+			IE_RSN_AKM_SUITE_8021X_SHA256 |
+			IE_RSN_AKM_SUITE_FT_OVER_8021X_SHA384);
+}
+
+static inline bool IE_CIPHER_IS_GCMP_CCMP(uint32_t cipher_suite)
+{
+	return cipher_suite & (IE_RSN_CIPHER_SUITE_CCMP |
+				IE_RSN_CIPHER_SUITE_CCMP_256 |
+				IE_RSN_CIPHER_SUITE_GCMP |
+				IE_RSN_CIPHER_SUITE_GCMP_256);
+}
+
+#define IE_GROUP_CIPHERS		\
+(					\
+	IE_RSN_CIPHER_SUITE_TKIP |	\
+	IE_RSN_CIPHER_SUITE_CCMP |	\
+	IE_RSN_CIPHER_SUITE_GCMP |	\
+	IE_RSN_CIPHER_SUITE_GCMP_256 |	\
+	IE_RSN_CIPHER_SUITE_CCMP_256	\
+)
+
+/*
+ * Since WEP is unsupported we can just use the group cipher list with
+ * "Use group cipher" appended
+ */
+#define IE_PAIRWISE_CIPHERS			\
+(						\
+	IE_GROUP_CIPHERS |			\
+	IE_RSN_CIPHER_SUITE_USE_GROUP_CIPHER	\
+)
 
 #define IE_LEN(ie) \
-	(ie) ? (ie)[1] + 2 : 0
+	((ie) ? (ie)[1] + 2 : 0)
 
 struct ie_tlv_iter {
 	unsigned int max;
@@ -393,6 +461,7 @@ struct ie_rsn_info {
 	bool spp_a_msdu_required:1;
 	bool pbac:1;
 	bool extended_key_id:1;
+	bool ocvc:1;
 	uint8_t num_pmkids;
 	const uint8_t *pmkids;
 	enum ie_rsn_cipher_suite group_management_cipher;
@@ -417,6 +486,7 @@ enum ie_bss_capability {
 
 struct ie_ft_info {
 	uint8_t mic_element_count;
+	bool rsnxe_used : 1;
 	uint8_t mic[24];
 	uint8_t anonce[32];
 	uint8_t snonce[32];
@@ -432,6 +502,8 @@ struct ie_ft_info {
 	uint8_t igtk_ipn[6];
 	uint8_t igtk_len;
 	uint8_t igtk[24];
+	bool oci_present:1;
+	uint8_t oci[3];
 };
 
 /* See chapter 8.4.2.47 for radio measurement capability details */
@@ -486,6 +558,14 @@ struct ie_fils_ip_addr_response_info {
 	uint8_t ipv6_lifetime;		/* Zero if not provided */
 };
 
+struct ie_owe_transition_info {
+	uint8_t bssid[6];
+	uint8_t ssid[32];
+	size_t ssid_len;
+	uint8_t oper_class;
+	uint8_t channel;
+};
+
 extern const unsigned char ieee_oui[3];
 extern const unsigned char microsoft_oui[3];
 extern const unsigned char wifi_alliance_oui[3];
@@ -539,6 +619,9 @@ unsigned char *ie_tlv_builder_finalize(struct ie_tlv_builder *builder,
 					size_t *out_len);
 
 uint32_t ie_rsn_cipher_suite_to_cipher(enum ie_rsn_cipher_suite suite);
+const char *ie_rsn_cipher_suite_to_string(enum ie_rsn_cipher_suite suite);
+
+uint32_t ie_rsn_akm_suite_to_akm(enum ie_rsn_akm_suite akm);
 
 int ie_parse_rsne(struct ie_tlv_iter *iter, struct ie_rsn_info *info);
 int ie_parse_rsne_from_data(const uint8_t *data, size_t len,
@@ -603,10 +686,11 @@ int ie_parse_roaming_consortium_from_data(const uint8_t *data, size_t len,
 int ie_build_roaming_consortium(const uint8_t *rc, size_t rc_len, uint8_t *to);
 
 int ie_parse_hs20_indication(struct ie_tlv_iter *iter, uint8_t *version,
-				uint16_t *pps_mo_id, uint8_t *domain_id);
+				uint16_t *pps_mo_id, uint8_t *domain_id,
+				bool *dgaf_disable);
 int ie_parse_hs20_indication_from_data(const uint8_t *data, size_t len,
 					uint8_t *version, uint16_t *pps_mo_id,
-					uint8_t *domain_id);
+					uint8_t *domain_id, bool *dgaf_disable);
 int ie_build_hs20_indication(uint8_t version, uint8_t *to);
 
 enum ie_rsnx_capability {
@@ -629,3 +713,10 @@ void ie_build_fils_ip_addr_response(
 
 int ie_parse_network_cost(const void *data, size_t len,
 				uint16_t *flags, uint16_t *level);
+
+int ie_parse_owe_transition(const void *data, size_t len,
+				struct ie_owe_transition_info *info);
+
+int ie_parse_oci(const void *data, size_t len, const uint8_t **oci);
+
+bool ie_validate_he_capabilities(const void *data, size_t len);
